import inspect
import sys
from operator import itemgetter

from django.utils import six, importlib


def inspect_class(cls):
    cls._instance = instance = cls()
    module = importlib.import_module(cls.__module__)
    members = sorted(inspect.getmembers(instance, inspect.ismethod),
                     key=itemgetter(0))
    public_members = [m for m in members if not m[0].startswith('_')]
    return public_members, module


class SettingsMeta(type):

    def __init__(cls, name, bases, attrs):
        public_members, module = inspect_class(cls)
        for member_name, member in public_members:
            setattr(module, member_name, member())


class ConfigMeta(type):

    def __new__(cls, name, bases, attrs):
        Class = super(ConfigMeta, cls).__new__(cls, name, bases, attrs)

        if name == 'NewBase':
            return Class

        public_members, module = inspect_class(Class)
        variables = ((k, val()) for (k, val) in public_members)
        result = ConfigResult(variables, Class)
        setattr(module, name, result)
        return result


class ConfigResult(dict):
    """
    Dict-like object which adds inheritance support.
    """
    def __new__(cls, *args, **kwargs):
        if len(args) == 2:
            # Used to create dict object
            return super(ConfigResult, cls).__new__(cls, *args, **kwargs)
        else:
            # Used as superclass
            name, bases, attrs = args
            bases = tuple(b.ConfigClass for b in bases if isinstance(b, ConfigResult))
            return ConfigMeta(name, bases, attrs)

    def __init__(self, *args, **kwargs):
        if len(args) == 2:
            # Is used as dict instance
            dict_arg, self._ConfigClass = args
            super(ConfigResult, self).__init__(dict_arg, **kwargs)
        else:
            # Is used as class
            pass

    @property
    def ConfigClass(self):
        return self._ConfigClass


class Settings(six.with_metaclass(SettingsMeta)):
    """
    Calls each public method of class and injects it's value into it's
    module's scope.
    """


class Config(six.with_metaclass(ConfigMeta)):
    """
    Calls each public method of class, constructs dictionary with `name-result`
    pairs and replaces class with it.
    """



import functools


def defaultargs(func):
    """
    Calls parameterized decorator automatically if no args were
    supplied.
    
    With : @mydecorator
    Without: @mydecorator()
    """
    @functools.wraps(func)
    def decorated(*args, **kwargs):
        if len(args) == 1 and callable(args[0]):
            return func()(args[0])
        else:
            return func(*args, **kwargs)
    return decorated



import functools
import os

from django.core.exceptions import ImproperlyConfigured

from .utils import defaultargs


@defaultargs
def from_env(key=None, through=None):
    """
    Gets environment variable by given key.
    If key is not present within environ, calls given function and raises
    :class:`ImproperlyConfigured` if it returns `None`.

    :param key: env. variable name, defaults to function's name
    :param through: callable should be applied to result
    """
    def decorator(func):
        @functools.wraps(func)
        def decorated(*args, **kwargs):
            try:
                value = get_env_setting(key or func.__name__)
            except ImproperlyConfigured as e:
                value = func(*args, **kwargs)
                if value is None:
                    raise e

            return through(value) if through else value

        return decorated
    return decorator


def get_env_setting(setting):
    """
    Gets the environment setting and raises exception if it isn't present.
    """
    try:
        return os.environ[setting]
    except KeyError:
        error_msg = "Set the %s env variable" % setting
        raise ImproperlyConfigured(error_msg)



from setuptools import setup, find_packages


setup(
    name = 'django-classsettings',
    version = '1.0.1',
    author='Dmitriy Ferens',
    author_email='ferensdima@gmail.com',
    description = 'Adds ability to define Django settings with classes'
                  '/functions and navigate each of them easily',
    long_description = open('README.rst').read(),
    license='MIT',
    platforms=['Any'],
    url='https://github.com/dferens/django-classsettings',
    download_url='https://pypi.python.org/pypi/django-classsettings',
    packages=find_packages(),
    package_data={'': ['LICENSE', 'README.rst']},
    include_package_data=True,
    install_requires=['Django>=1.4,<1.7'],
    zip_safe=True,
    classifiers=(
        'Development Status :: 2 - Pre-Alpha',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ),
)
